
import org.deeplearning4j.rl4j.learning.sync.qlearning.QLearning;
import org.deeplearning4j.rl4j.learning.sync.qlearning.discrete.QLearningDiscreteDense;
import org.deeplearning4j.rl4j.network.dqn.DQNFactoryStdDense;
import org.deeplearning4j.rl4j.policy.DQNPolicy;
import org.deeplearning4j.rl4j.space.DiscreteSpace;
import org.deeplearning4j.rl4j.space.Encodable;
import org.deeplearning4j.rl4j.mdp.MDP;
import org.deeplearning4j.rl4j.util.DataManager;
import org.nd4j.linalg.learning.config.RmsProp;
import java.io.IOException;
import java.util.logging.Logger;

public class MoveToOrign {
	public static void main(String[] args) throws IOException {
		// DataManagerはデータを複数のアプリセッションで利用できるようにする
		DataManager manager = new DataManager(true);
		// Q学習のハイパーパラメータを定義
		QLearning.QLConfiguration MOVE_QL = QLearning.QLConfiguration.builder()
			.seed(1)						// Random seed (for reproducability)
			.maxEpochStep(300)				// Max step By epoch
			.maxStep(20000)					// Max step
			.expRepMaxSize(20000)			// Max size of experience replay
			.batchSize(128)					// size of batches
			.targetDqnUpdateFreq(500)		// target update (hard)
            .updateStart(100)				// num step noop warmup
			.rewardFactor(1)				// reward scaling
			.gamma(0.99)					// gamma
			.errorClamp(1.0)				// td-error clipping
			.minEpsilon(0.1f)				// min epsilon
			.epsilonNbStep(1000)			// num step for eps greedy anneal
			.doubleDQN(true)				// double DQN
			.build();
        // ネットワークの構成を定義
		DQNFactoryStdDense.Configuration MOVE_NET = DQNFactoryStdDense.Configuration.builder()
			.l2(0)
            .updater(new RmsProp(0.001))
            .numHiddenNodes(16)				// 隠れ層のノード数
            .numLayer(3)					// 隠れ層の層数
			.build();
		// 訓練用にMoveMDPクラスのインスタンスmdp1を生成
		MDP<Encodable, Integer, DiscreteSpace> mdp1 = new MoveMDP(false);
		// mdp1と訓練用パラメータで訓練を定義
		QLearningDiscreteDense<Encodable> dql = new QLearningDiscreteDense(
			mdp1, MOVE_NET, MOVE_QL, manager);
		// 訓練を実行
		dql.train();
        // 最終のpolicyを保存
		dql.getPolicy().save("policy.zip");
		mdp1.close();
		// テスト用にMoveMDPクラスのインスタンスmdp2を生成
		MDP<Encodable, Integer, DiscreteSpace> mdp2 = new MoveMDP(true);
		// 保存したpolicyをロード
		DQNPolicy<Encodable>  pol = DQNPolicy.load("policy.zip");
		// テストを実行
		double rewards = 0;
		for (int i = 0; i < 10; i++) {
			double reward = pol.play(mdp2);
			rewards += reward;
			Logger.getAnonymousLogger().info("Reward: " + reward);
        }
		// 最後のepisodeのposLogを残すためにリセットする
		mdp2.reset();
		Logger.getAnonymousLogger().info("average: " + rewards/10.);
		mdp2.close();
    }
}
